// Copyright 2017 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

package com.qpc.qpc_admob;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.util.SparseArray;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;

import com.bytedance.sdk.openadsdk.AdSlot;
import com.bytedance.sdk.openadsdk.TTAdConstant;
import com.bytedance.sdk.openadsdk.TTAdDislike;
import com.bytedance.sdk.openadsdk.TTAdManager;
import com.bytedance.sdk.openadsdk.TTAdNative;
import com.bytedance.sdk.openadsdk.TTAdSdk;
import com.bytedance.sdk.openadsdk.TTAppDownloadListener;
import com.bytedance.sdk.openadsdk.TTBannerAd;
import com.bytedance.sdk.openadsdk.TTImage;
import com.bytedance.sdk.openadsdk.TTInteractionAd;
import com.bytedance.sdk.openadsdk.TTNativeAd;
import com.bytedance.sdk.openadsdk.TTNativeExpressAd;
import com.bytedance.sdk.openadsdk.TTNativeExpressAd.ExpressAdInteractionListener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.MethodChannel;

abstract class QsjMobileAd extends MobileAd implements ExpressAdInteractionListener, TTAdNative.InteractionAdListener, TTNativeExpressAd.AdInteractionListener, TTAdNative.NativeExpressAdListener,TTAppDownloadListener {
    private static final String TAG = "flutter";

    TTAdNative mTTAdNative;
    TTInteractionAd ttInteractionAd;
    FrameLayout content ;



    protected QsjMobileAd(int id, Activity activity, MethodChannel channel) {
     super(id,activity,channel);
    }

    static QsjBanner createBanner(Integer id, String adUnitId, int width, int height, Activity activity, MethodChannel channel) {
        MobileAd ad = getAdForId(id);
//        Intent intent = new Intent(activity,QsjMobileBannerAdActivity.class);
//        activity.startActivity(intent);
        return (ad != null) ? (QsjBanner) ad : new QsjBanner(id,width,height,activity,channel);

    }

    static QsjNativeExpressAd createExpressBanner(Integer id, String adUnitId, int width, int height, Activity activity, MethodChannel channel) {
        MobileAd ad = getAdForId(id);
//        Intent intent = new Intent(activity,QsjMobileBannerAdActivity.class);
//        activity.startActivity(intent);
        return (ad != null) ? (QsjNativeExpressAd) ad : new QsjNativeExpressAd(id,width,height,activity,channel);

    }

    static QsjInterstitial createInterstitial(Integer id, Activity activity, MethodChannel channel) {
        MobileAd ad = getAdForId(id);
//        Intent intent = new Intent(activity,QsjMobileBannerAdActivity.class);
//        activity.startActivity(intent);
        return (ad != null) ? (QsjInterstitial) ad : new QsjInterstitial(id, activity, channel);
    }


    @Override
    public void onAdClicked(View view, int type) {
//    TToast.show(mContext, "广告被点击");
        channel.invokeMethod("onAdClicked", argumentsMap());
    }

    @Override
    public void onAdShow(View view, int type) {
//    TToast.show(mContext, "广告展示");
        boolean statusWasPending = status == Status.PENDING;
        status = Status.LOADED;
        channel.invokeMethod("onAdLoaded", argumentsMap());
        if (statusWasPending) show();

    }

    @Override
    public void onAdDismiss() {
        Log.d(TAG, "插屏广告消失");
        TToast.show(activity, "广告消失");
    }

    @Override
    public void onRenderFail(View view, String msg, int code) {
        Log.w(TAG, msg + code);
        status = Status.FAILED;
        channel.invokeMethod("onAdFailedToLoad", argumentsMap("errorMsg",msg,"errorCode", code));
    }



    @Override
    public void onIdle() {
        TToast.show(activity, "点击开始下载", Toast.LENGTH_LONG);
    }

    @Override
    public void onDownloadActive(long totalBytes, long currBytes, String fileName, String appName) {

        TToast.show(activity, "下载中，点击暂停", Toast.LENGTH_LONG);

    }

    @Override
    public void onDownloadPaused(long totalBytes, long currBytes, String fileName, String appName) {
        TToast.show(activity, "下载暂停，点击继续", Toast.LENGTH_LONG);
    }

    @Override
    public void onDownloadFailed(long totalBytes, long currBytes, String fileName, String appName) {
        TToast.show(activity, "下载失败，点击重新下载", Toast.LENGTH_LONG);
    }

    @Override
    public void onInstalled(String fileName, String appName) {
        TToast.show(activity, "安装完成，点击图片打开", Toast.LENGTH_LONG);
    }

    @Override
    public void onDownloadFinished(long totalBytes, String fileName, String appName) {
        TToast.show(activity, "点击安装", Toast.LENGTH_LONG);
    }


    @Override
    public void onError(int code, String message) {
        TToast.show(activity, "load error : " + code + ", " + message);
    }

    @Override
    public void onInteractionAdLoad(TTInteractionAd ttInteractionAd) {
        TToast.show(activity, "type:  " + ttInteractionAd.getInteractionType());
        ttInteractionAd.setAdInteractionListener((TTInteractionAd.AdInteractionListener) this);
        ttInteractionAd.setDownloadListener(this);
        this.ttInteractionAd = ttInteractionAd;
        ttInteractionAd.showInteractionAd(activity);

    }

    @Override
    public void onNativeExpressAdLoad(List<TTNativeExpressAd> ads) {
        TToast.show(activity, "load ads : " + ads.size() );
        if (ads == null || ads.size() == 0){
            TToast.show(activity, "load ads : " + 0 );
            return;
        }
        TToast.show(activity, "load ads : " + ads.size() );
        TTNativeExpressAd mTTAd = ads.get(0);
        View view = mTTAd.getExpressAdView();
//        content = new LinearLayout(activity);
        content.setId(id);
//        content.setOrientation(LinearLayout.VERTICAL);
//        content.setGravity(anchorType);
        content.setMinimumHeight(100);
        content.setMinimumWidth(300);

        view.bringToFront();
        view.setActivated(true);
        view.setVisibility(View.VISIBLE);
        content.addView(view);

        final float scale = activity.getResources().getDisplayMetrics().density;
//
//        int left = horizontalCenterOffset > 0 ? (int) (horizontalCenterOffset * scale) : 0;
//        int right =
//                horizontalCenterOffset < 0 ? (int) (Math.abs(horizontalCenterOffset) * scale) : 0;
//        if (anchorType == Gravity.BOTTOM) {
//            content.setPadding(left, 0, right, (int) (anchorOffset * scale));
//        } else {
//            content.setPadding(left, (int) (anchorOffset * scale), right, 0);
//        }

        activity.addContentView(
                content,
                new ViewGroup.LayoutParams(
                        ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));

        mTTAd.setSlideIntervalTime(30*1000);//设置轮播间隔 ms,不调用则不进行轮播展示
//        mTTAd.setExpressInteractionListener(this);
//        mTTAd.setDownloadListener(this);
        mTTAd.render();//调用render开始渲染广告


    }

    @Override
    public void onRenderSuccess(View view, float width, float height) {
        //返回view的宽高 单位 dp
        TToast.show(activity, "渲染成功haha");
        //在渲染成功回调时展示广告，提升体验
//    mExpressContainer.removeAllViews();
//    mExpressContainer.addView(view)

        view.bringToFront();



    }

    static class QsjBanner extends QsjMobileAd {

        private  int width;
        private int height;
        private Context mContext;
        private Button mButtonDownload;
        private Button mButtonLandingPage;
        private Button mCreativeButton;
        private FrameLayout mBannerContainer;



        private QsjBanner(Integer id, int width, int height, Activity activity, MethodChannel channel) {

            super(id, activity, channel);
            this.width=width;
            this.height=height;
            this.mContext = activity;


        }

        @Override
        void load(String adUnitId, Map<String, Object> targetingInfo) {

            //step5:请求广告
            TTAdManager ttAdManager =  TTAdSdk.getAdManager();
            //step2:(可选，强烈建议在合适的时机调用):申请部分权限，如read_phone_state,防止获取不了imei时候，下载类广告没有填充的问题。
            ttAdManager.requestPermissionIfNecessary(activity);
            //step3:创建TTAdNative对象,用于调用广告请求接口
            mTTAdNative = ttAdManager.createAdNative(activity);

            //step4:创建广告请求参数AdSlot,具体参数含义参考文档
            final AdSlot adSlot = new AdSlot.Builder()
                    .setCodeId(adUnitId)
                    .setSupportDeepLink(true)
                    .setImageAcceptedSize(600, 200)
                    .setNativeAdType(AdSlot.TYPE_BANNER) //请求原生广告时候，请务必调用该方法，设置参数为TYPE_BANNER或TYPE_INTERACTION_AD
                    .setAdCount(1)
                    .build();



                    //绑定原生广告的数据


//            step5:请求广告，对请求回调的广告作渲染处理
            mTTAdNative.loadNativeAd(adSlot, new TTAdNative.NativeAdListener() {
                @Override
                public void onError(int code, String message) {
                    TToast.show(activity, "load error 555: " + code + ", " + message);
                }

                @Override
                public void onNativeAdLoad(List<TTNativeAd> ads) {
                    try {

                        if (ads.get(0) == null) {
                            return;
                        }
                        View bannerView = LayoutInflater.from(activity).inflate(R.layout.native_ad, null, false);
                        if (bannerView == null) {
                            return;
                        }
                        LinearLayout content = new LinearLayout(activity);
                        content.setId(id);
                        content.setOrientation(LinearLayout.VERTICAL);
                        content.setGravity(anchorType);
                        content.addView(bannerView);
                        final float scale = activity.getResources().getDisplayMetrics().density;

                        int left = horizontalCenterOffset > 0 ? (int) (horizontalCenterOffset * scale) : 0;
                        int right =
                                horizontalCenterOffset < 0 ? (int) (Math.abs(horizontalCenterOffset) * scale) : 0;
                        if (anchorType == Gravity.BOTTOM) {
                            content.setPadding(left, 0, right, (int) (anchorOffset * scale));
                        } else {
                            content.setPadding(left, (int) (anchorOffset * scale), right, 0);
                        }

                        //绑定原生广告的数据
                        setAdData(bannerView, ads.get(0));

                        activity.addContentView(
                                content,
                                new ViewGroup.LayoutParams(
                                        ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));

                    }catch (Exception e) {
                        System.out.println(e);
                        TToast.show(activity, "load error 888: " + e.getLocalizedMessage());

                    }

                }
            });
        }

        @SuppressWarnings("RedundantCast")
        private void setAdData(View nativeView, TTNativeAd nativeAd) {
            ((TextView) nativeView.findViewById(R.id.tv_native_ad_title)).setText(nativeAd.getTitle());
            ((TextView) nativeView.findViewById(R.id.tv_native_ad_desc)).setText(nativeAd.getDescription());
            ImageView imgDislike = nativeView.findViewById(R.id.img_native_dislike);
            if (nativeAd.getImageList() != null && !nativeAd.getImageList().isEmpty()) {
                TTImage image = nativeAd.getImageList().get(0);
                if (image != null && image.isValid()) {
                    ImageView im = nativeView.findViewById(R.id.iv_native_image);
                    com.bumptech.glide.Glide.with(activity).load(image.getImageUrl()).into(im);
                }
            }
            TTImage icon = nativeAd.getIcon();
            if (icon != null && icon.isValid()) {
                ImageView im = nativeView.findViewById(R.id.iv_native_icon);
                com.bumptech.glide.Glide.with(activity).load(icon.getImageUrl()).into(im);
            }
            mCreativeButton = (Button) nativeView.findViewById(R.id.btn_native_creative);
            //可根据广告类型，为交互区域设置不同提示信息
            switch (nativeAd.getInteractionType()) {
                case TTAdConstant.INTERACTION_TYPE_DOWNLOAD:
                    //如果初始化ttAdManager.createAdNative(getApplicationContext())没有传入activity 则需要在此传activity，否则影响使用Dislike逻辑
                    nativeAd.setActivityForDownloadApp(activity);
                    mCreativeButton.setVisibility(View.VISIBLE);
//                    nativeAd.setDownloadListener(mDownloadListener); // 注册下载监听器
                    break;
                case TTAdConstant.INTERACTION_TYPE_DIAL:
                    mCreativeButton.setVisibility(View.VISIBLE);
                    mCreativeButton.setText("立即拨打");
                    break;
                case TTAdConstant.INTERACTION_TYPE_LANDING_PAGE:
                case TTAdConstant.INTERACTION_TYPE_BROWSER:
                    mCreativeButton.setVisibility(View.VISIBLE);
                    mCreativeButton.setText("查看详情");
                    break;
                default:
                    mCreativeButton.setVisibility(View.GONE);
                    TToast.show(mContext, "交互类型异常");
            }

            //可以被点击的view, 也可以把nativeView放进来意味整个广告区域可被点击
            List<View> clickViewList = new ArrayList<>();
            clickViewList.add(nativeView);

            //触发创意广告的view（点击下载或拨打电话）
            List<View> creativeViewList = new ArrayList<>();
            //如果需要点击图文区域也能进行下载或者拨打电话动作，请将图文区域的view传入
            //creativeViewList.add(nativeView);
            creativeViewList.add(mCreativeButton);

            //重要! 这个涉及到广告计费，必须正确调用。convertView必须使用ViewGroup。
            nativeAd.registerViewForInteraction((ViewGroup) nativeView, clickViewList, creativeViewList, imgDislike, new TTNativeAd.AdInteractionListener() {
                @Override
                public void onAdClicked(View view, TTNativeAd ad) {
                    if (ad != null) {
                        TToast.show(mContext, "广告" + ad.getTitle() + "被点击");
                    }
                }

                @Override
                public void onAdCreativeClick(View view, TTNativeAd ad) {
                    if (ad != null) {
                        TToast.show(mContext, "广告" + ad.getTitle() + "被创意按钮被点击");
                    }
                }

                @Override
                public void onAdShow(TTNativeAd ad) {
                    if (ad != null) {
                        TToast.show(mContext, "广告" + ad.getTitle() + "展示");
                    }
                }
            });

        }

        @Override
        void show() {
            if (status == Status.LOADING) {
                status = Status.PENDING;
                return;
            }
            if (status != Status.LOADED) return;

        }

        @Override
        void dispose() {
            super.dispose();


            View contentView = activity.findViewById(id);
            if (contentView == null || !(contentView.getParent() instanceof ViewGroup)) return;

            ViewGroup contentParent = (ViewGroup) (contentView.getParent());
            contentParent.removeView(contentView);
            mTTAdNative=null;
        }
    }

    static class QsjInterstitial extends QsjMobileAd{

        private QsjInterstitial(int id, Activity activity, MethodChannel channel) {
            super(id, activity, channel);
            //step5:请求广告
            TTAdManager ttAdManager =  TTAdSdk.getAdManager();
            //step2:(可选，强烈建议在合适的时机调用):申请部分权限，如read_phone_state,防止获取不了imei时候，下载类广告没有填充的问题。
            ttAdManager.requestPermissionIfNecessary(activity);
            //step3:创建TTAdNative对象,用于调用广告请求接口
            mTTAdNative = ttAdManager.createAdNative(activity);
        }

        @Override
        void load(String adUnitId, Map<String, Object> targetingInfo) {

        }

        @Override
        void show() {

        }

        // It is not possible to hide/remove/destroy an AdMob interstitial Ad.
    }
}
